# -*- coding: utf-8 -*-
from plone import api
from plone.app.testing import logout
from plone.app.ldap.config import PROJECTNAME
from plone.app.ldap.testing import INTEGRATION_TESTING

import pkg_resources
import unittest

qi_dist = pkg_resources.get_distribution("Products.CMFQuickInstallerTool")
if qi_dist.parsed_version > pkg_resources.parse_version("3.0.9"):
    # This version uses the uninstall profile.
    SKIP_UNINSTALL_TEST = False
else:
    SKIP_UNINSTALL_TEST = True


class ControlPanelTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.controlpanel = self.portal['portal_controlpanel']

    def test_controlpanel_has_view(self):
        view = api.content.get_view(
            u'ldap-controlpanel', self.portal, self.request)
        self.assertTrue(view())

    def test_controlpanel_view_is_protected(self):
        from AccessControl import Unauthorized
        logout()
        with self.assertRaises(Unauthorized):
            self.portal.restrictedTraverse('@@ldap-controlpanel')

    def test_controlpanel_installed(self):
        actions = [
            a.getAction(self)['id'] for a in self.controlpanel.listActions()]
        self.assertIn('ldap', actions)

    @unittest.skipIf(SKIP_UNINSTALL_TEST,
                     "uninstall not supported with this old quick installer")
    def test_controlpanel_removed_on_uninstall(self):
        qi = self.portal['portal_quickinstaller']

        with api.env.adopt_roles(['Manager']):
            qi.uninstallProducts(products=[PROJECTNAME])

        actions = [
            a.getAction(self)['id'] for a in self.controlpanel.listActions()]
        self.assertNotIn('ldap', actions)



# -*- coding: utf-8 -*-
from plone.app.ldap.engine.interfaces import ILDAPConfiguration
from plone.app.ldap.engine.schema import LDAPProperty
from plone.app.ldap.engine.server import LDAPServer
from plone.app.ldap.engine.storage import LDAPSchema
from plone.app.ldap.engine.storage import LDAPServerStorage
from plone.app.ldap.ploneldap.util import configureLDAPSchema
from plone.app.ldap.ploneldap.util import configureLDAPServers
from plone.app.ldap.ploneldap.util import createLDAPPlugin
from plone.app.ldap.ploneldap.util import getLDAPPlugin
from plone.app.ldap.ploneldap.util import getPAS
from six import StringIO
from xml.dom.minidom import parseString
from zope.component import getUtility
from zope.pagetemplate.pagetemplatefile import PageTemplateFile

import ast
import logging


ldap_props = ['_login_attr',
              '_uid_attr',
              '_rdnattr',
              'users_base',
              'users_scope',
              '_local_groups',
              '_implicit_mapping',
              '_groups_mappings',
              'groups_base',
              'groups_scope',
              '_binduid',
              '_bindpwd',
              '_binduid_usage',
              'read_only',
              '_user_objclasses',
              '_extra_user_filter',
              '_pwd_encryption',
              '_roles']

_FILENAME = 'ldap_plugin.xml'

update = True


class LDAPPluginExportImport:
    """In- and Exporter for LDAP-PAS-Plugin
    """

    def exportData(self, context, out):
        template = PageTemplateFile('xml/%s' % _FILENAME, globals())  # .__of__(context.getSite())
        info = self._getExportInfo(context)
        if info:
            context.writeDataFile('%s' % _FILENAME, template(info=info).encode('utf-8'), 'text/xml')
            print >> out, "GenericSetup Configuration for ldap exported"

    def getTypeStr(self, value):
        val_type = 'str'
        if isinstance(value, list):
            val_type = 'list'
        # check for bool before int as a bool is also an instance of int
        elif isinstance(value, bool):
            val_type = 'bool'
        elif isinstance(value, int):
            val_type = 'int'
        return val_type

    def _getExportInfo(self, context):
        info = []
        portal = context.getSite()
        mt = getattr(portal, 'acl_users')
        for obj in mt.objectValues():
            if obj.meta_type in ['Plone LDAP plugin', 'LDAP Multi Plugin', 'Plone Active Directory plugin', 'ActiveDirectory Multi Plugin']:
                interfaces = []
                for p_info in obj.plugins.listPluginTypeInfo():
                    interface = p_info['interface']
                    actives = obj.plugins.listPlugins(interface)
                    act_ids = [x[0] for x in actives]
                    if obj.getId() in act_ids:
                        interfaces.append(p_info['id'])

                plugin_props = obj.propertyMap()
                for prop in plugin_props:
                    value = obj.getProperty(prop['id'])
                    if prop['type'] in ['string', 'int']:
                        prop['value'] = obj.getProperty(prop['id'])
                plugin_props = [i for i in plugin_props if 'value' in i.keys()]

                c_info = {
                    'meta_type': obj.meta_type,
                    'plugin_props': plugin_props,
                    'interfaces': interfaces,
                    'properties': [],
                    'schema': [],
                    'servers': [],
                    'id': obj.getId(),
                    'title': obj.title}
                uf = getattr(obj, 'acl_users')
                for prop in ldap_props:
                    value = uf.getProperty(prop)
                    val_type = self.getTypeStr(value)
                    if val_type != 'list':
                        value = [value]
                    c_info['properties'].append({'id': prop, 'type': val_type, 'value': value})
                for server in uf.getServers():
                    c_server = {'content': []}
                    for key in server.keys():
                        c_server['content'].append({'id': key, 'value': server[key], 'type': self.getTypeStr(server[key])})
                    c_info['servers'].append(c_server)
                schema = uf.getSchemaConfig()
                for key in schema.keys():
                    a_item = {'id': key, 'content': []}
                    for subkey in schema[key]:
                        s_item = {'id': subkey, 'value': schema[key][subkey], 'type': self.getTypeStr(schema[key][subkey])}
                        a_item['content'].append(s_item)
                    c_info['schema'].append(a_item)
                info.append(c_info)

        return info

    def importData(self, context, out):
        logger = context.getLogger('ldapsettings')
        xml = context.readDataFile(_FILENAME)
        if xml is None:
            logger.info('Nothing to import.')
            return

        portal = context.getSite()
        pas = getattr(portal, 'acl_users')
        dom = parseString(xml)
        root = dom.documentElement

        for plugin in root.getElementsByTagName('ldapplugin'):
            self.extractData(plugin, pas, out)

    def extractData(self, root, pas, out):
        plug_id = str(root.getAttribute('id'))
        update = root.getAttribute('update') == 'True'
        meta_type = root.getAttribute('meta_type')

        settings = {}
        interfaces = []
        plugin_props = []
        for prop in root.getElementsByTagName('plugin_property'):
            p_type = prop.getAttribute('type')
            p_id = prop.getAttribute('id')
            value = prop.getAttribute('value')
            if p_type == 'int':
                value = int(value)
            if p_type == 'string':
                value = str(value)
            plugin_props.append({'id': p_id, 'type': p_type, 'value': value})

        for iface in root.getElementsByTagName('interface'):
            interfaces.append(iface.getAttribute('value'))

        caches = list()
        for node in root.getElementsByTagName('cache'):
            caches.append(node.getAttribute('value'))

        if len(caches) > 1:
            raise ValueError('You can not define multiple <cache> properties')

        cache = ''
        if len(caches):
            cache = caches[0]

        for prop in root.getElementsByTagName('property'):
            type = prop.getAttribute('type')
            values = []
            for v in prop.getElementsByTagName('item'):
                values.append(v.getAttribute('value'))
            id = prop.getAttribute('id')
            if type == 'list':
                # values are unicode strings
                # _user_objclasses and _roles need to be strings
                if id in ['_user_objclasses', '_roles']:
                    value = [item.encode('utf8') for item in values]
                else:
                    value = values
            else:
                value = values[0]
            if type == 'int':
                value = int(value)
            if type == 'bool':
                value = (value.lower() != 'false' and 1 or 0)
            settings[id] = value
        schema = {}
        for schemanode in root.getElementsByTagName('schema'):
            for attr in schemanode.getElementsByTagName('attr'):
                c_id = attr.getAttribute('id')
                c_attr = {}
                for item in attr.getElementsByTagName('item'):
                    if item.getAttribute('value') != 'False':
                        c_attr[str(item.getAttribute('id'))] = str(item.getAttribute('value'))
                    else:
                        c_attr[str(item.getAttribute('id'))] = False
                schema[str(c_id)] = c_attr
        servers = []
        for server in root.getElementsByTagName('server'):
            c_server = {'update': (server.getAttribute('update') == 'True'),
                        'delete': (server.getAttribute('delete') == 'True')}
            for item in server.getElementsByTagName('item'):
                value = item.getAttribute('value')
                type = item.getAttribute('type')
                id = item.getAttribute('id')
                if type == 'int':
                    value = int(value)
                c_server[id] = value
            servers.append(c_server)

        # always update if it doesn't exist
        if plug_id not in pas.objectIds():
            update = True

        if update:
            # delete existing LDAP plug-in
            if plug_id in pas.objectIds():
                try:
                    plugin = getLDAPPlugin()
                    pas = getPAS()
                    pas.manage_delObjects([plugin.getId()])
                except KeyError:
                    # pass
                    """
                    There are two reasons to not pass here. First, if we pass
                    and go to recreate later and both plugins have the same it, it
                    will error out for the id already existing. Second, if they
                    don't have the same id but have the same settings, they will then
                    in practice (if its set up correct) have duplicate users, which
                    will subsequently break any group or role lookups which assert
                    on the duplicate users. I don't see any tests on this so if there
                    is an argument to leave this as a pass let me know.
                    """
                    logging.error("There is an ldap multi plugin in your "+
                        "system (%s) that is not managed "%plug_id +
                        "by this generic setup script. To have everything "+
                        "managed by GS, please delete and " +
                        "reinstall or set update=False in your ldap_plugin.xml"+
                        " root.")
                    logging.error("Installing LDAP Plugin with GS failed")
                    return

            # base configuration
            config = getUtility(ILDAPConfiguration)
            if meta_type in [u"Plone Active Directory plugin",
                             u"ActiveDirectory Multi Plugin"]:
                config.ldap_type = u"AD"
            else:
                config.ldap_type = u"LDAP"

            config.login_attribute = settings['_login_attr']
            config.userid_attribute = settings['_uid_attr']
            config.rdn_attribute = settings['_rdnattr']
            config.user_base = settings['users_base']
            config.user_scope = settings['users_scope']
            config.group_base = settings['groups_base']
            config.group_scope = settings['groups_scope']
            config.bind_dn = settings['_binduid']
            config.bind_password = settings['_bindpwd']
            config.user_object_classes = ','.join(settings['_user_objclasses'])
            config.extra_user_filter = settings['_extra_user_filter']
            config.password_encryption = settings['_pwd_encryption']
            config.default_user_roles = ','.join(settings['_roles'])
            config.implicit_mapping = settings['_implicit_mapping']
            config.local_groups = settings['_local_groups']
            try:
                config.group_mappings = ast.literal_eval(settings['_groups_mappings'])
            except (ValueError, SyntaxError, KeyError):
                config.group_mappings = {}
            config.read_only = settings['read_only']
            config.activated_plugins = interfaces
            config.cache = cache

            # servers
            config.servers = LDAPServerStorage()
            for server in servers:
                obj = LDAPServer(server=server['host'],
                                 connection_type=(server['protocol'] == 'ldaps'),
                                 connection_timeout=server['conn_timeout'],
                                 operation_timeout=server['op_timeout'],
                                 enabled=True)
                config.servers.addItem(obj)

            # schema
            config.schema = LDAPSchema()
            for property in schema.itervalues():
                obj = LDAPProperty(ldap_name=property.get('ldap_name', ''),
                                   plone_name=property.get('public_name', ''),
                                   description=property.get('friendly_name', ''),
                                   multi_valued=property.get('multivalued', False),
                                   binary=property.get('binary', False))
                config.schema.addItem(obj)
            # recreate new LDAP plug-in
            createLDAPPlugin(plug_id)
            configureLDAPServers()
            configureLDAPSchema()


def exportLDAPSettings(context):
    exporter = LDAPPluginExportImport()
    out = StringIO()
    exporter.exportData(context, out)
    logger = context.getLogger('ldapsettings')
    logger.info(out.getvalue())


def importLDAPSettings(context):
    importer = LDAPPluginExportImport()
    out = StringIO()
    importer.importData(context, out)
    logger = context.getLogger('ldapsettings')
    logger.info(out.getvalue())



# -*- coding: utf-8 -*-
from zope.interface import implementer
from zope.schema.vocabulary import SimpleVocabulary
from zope.schema.vocabulary import SimpleTerm
from zope.component import getUtility
from plone.app.ldap.engine.interfaces import ILDAPConfiguration
import ldap

from zope.schema.interfaces import IVocabularyFactory


@implementer(IVocabularyFactory)
class LDAPServerTypeVocabulary(object):
    """Vocabulary factory for LDAP server types.
    """

    def __call__(self, context):
        return SimpleVocabulary([
            SimpleTerm(u"LDAP", u"LDAP"),
            SimpleTerm(u"AD", u"Active Directory"),
            ])

LDAPServerTypeVocabularyFactory = LDAPServerTypeVocabulary()


@implementer(IVocabularyFactory)
class LDAPConnectionTypeVocabulary(object):
    """Vocabulary factory for LDAP connection types.
    """

    def __call__(self, context):
        return SimpleVocabulary([
            SimpleTerm(0, u"LDAP"),
            SimpleTerm(1, u"LDAP over SSL"),
            SimpleTerm(2, u"LDAP over IPC"),
            ])

LDAPConnectionTypeVocabularyFactory = LDAPConnectionTypeVocabulary()


@implementer(IVocabularyFactory)
class LDAPScopeVocabulary(object):
    """Vocabulary factory for LDAP search scopes.
    """

    def __call__(self, context):
        return SimpleVocabulary([
            SimpleTerm(ldap.SCOPE_ONELEVEL, u"one level"),
            SimpleTerm(ldap.SCOPE_SUBTREE, u"subtree"),
            ])

LDAPScopeVocabularyFactory = LDAPScopeVocabulary()


@implementer(IVocabularyFactory)
class LDAPAttributesVocabulary(object):
    """Vocabulary factory for LDAP attributes.
    """

    def __call__(self, context):
        config=getUtility(ILDAPConfiguration)
        attributes=[(a.ldap_name, a.__name__) for a in config.schema.values()]
        return SimpleVocabulary.fromItems(sorted(attributes))

LDAPAttributesVocabularyFactory = LDAPAttributesVocabulary()


@implementer(IVocabularyFactory)
class LDAPSingleValueAttributesVocabulary(object):
    """Vocabulary factory for LDAP attributes.
    """

    def __call__(self, context):
        config=getUtility(ILDAPConfiguration)
        attributes=[(a.ldap_name, a.__name__) for a in config.schema.values()
                    if not a.multi_valued]
        return SimpleVocabulary.fromItems(sorted(attributes))

LDAPSingleValueAttributesVocabularyFactory = LDAPSingleValueAttributesVocabulary()


@implementer(IVocabularyFactory)
class LDAPPasswordEncryptionVocabulary(object):
    """Vocabulary factory for LDAP Password encryption.
    """

    def __call__(self, context):
        return SimpleVocabulary([
            SimpleTerm('crypt', u"crypt"),
            SimpleTerm('SHA', u"SHA"),
            SimpleTerm('SSHA', u"SSHA"),
            SimpleTerm('md5', u"md5"),
            SimpleTerm('clear', u"clear"),
            ])

LDAPPasswordEncryptionVocabularyFactory = LDAPPasswordEncryptionVocabulary()



# -*- coding: utf-8 -*-
from zope.interface import implementer
from zope.container.ordered import OrderedContainer
from zope.container.interfaces import INameChooser
from plone.app.ldap.engine.interfaces import ILDAPServerStorage
from plone.app.ldap.engine.interfaces import ILDAPSchema
from plone.app.ldap.engine.interfaces import ILDAPConfiguration
from BTrees.OOBTree import OOBTree
from ldap import SCOPE_SUBTREE
from plone.app.ldap.engine.schema import LDAPProperty


@implementer(ILDAPConfiguration)
class LDAPConfiguration(OrderedContainer):

    ldap_type = u"LDAP"
    rdn_attribute = "uid"
    userid_attribute = "uid"
    login_attribute = "uid"
    required_attributes = []

    user_object_classes = "pilotPerson"

    bind_dn = ""
    bind_password = ""
    user_base = ""
    user_scope = SCOPE_SUBTREE
    group_base = ""
    group_scope = SCOPE_SUBTREE
    password_encryption = "crypt"
    default_user_roles = "Member"
    extra_user_filter = ""
    local_groups = False
    implicit_mapping = False
    group_mappings = {}
    read_only = False
    activated_plugins = []
    cache = ''

    def __init__(self):
        self.servers=LDAPServerStorage()
        self.schema=LDAPSchema()

        self.schema.addItem(LDAPProperty(
            ldap_name="uid", description=u"User id"))
        self.schema.addItem(LDAPProperty(
            ldap_name="mail", plone_name="email",
            description=u"Email address"))
        self.schema.addItem(LDAPProperty(
            ldap_name="cn", plone_name="fullname",
            description=u"Canonical Name"))
        self.schema.addItem(LDAPProperty(
            ldap_name="sn", description=u"Surname (unused)"))


class LDAPContainer(OrderedContainer):
    """Base class for our containers.
    """
    def __init__(self):
        OrderedContainer.__init__(self)
        self._data=OOBTree()

    def addItem(self, item):
        chooser=INameChooser(self)
        item_id = getattr(item, 'ldap_name', None) or chooser.chooseName(None, item)
        self[item_id]=item


@implementer(ILDAPServerStorage)
class LDAPServerStorage(LDAPContainer):
    """A container for LDAP servers.
    """


@implementer(ILDAPSchema)
class LDAPSchema(LDAPContainer):
    """A container for LDAP properties.
    """



# -*- coding: utf-8 -*-
from zope.component import adapter
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from plone.app.ldap.engine.interfaces import ILDAPBinding
from plone.app.ldap.ploneldap.util import guaranteePluginExists
from plone.app.ldap.ploneldap.util import getLDAPPlugin


@adapter(ILDAPBinding, IObjectModifiedEvent)
def HandleModified(config, event):
    if guaranteePluginExists():
        # A new fully configured plugin has been created, so we do not
        # need to do anything anymore.
        return

    luf=getLDAPPlugin()._getLDAPUserFolder()
    luf.manage_edit(
            title="Plone managed LDAP",
            login_attr=str(config.schema[config.login_attribute].ldap_name),
            uid_attr=str(config.schema[config.userid_attribute].ldap_name),
            rdn_attr=str(config.schema[config.rdn_attribute].ldap_name),
            users_base=config.user_base or "",
            users_scope=config.user_scope,
            groups_base=config.group_base or "",
            groups_scope=config.group_scope,
            binduid=str(config.bind_dn) or "",
            bindpwd=str(config.bind_password) or "",
            encryption=config.password_encryption,
            roles=config.default_user_roles or "",
            read_only=config.read_only,
            obj_classes=config.user_object_classes)



# -*- coding: utf-8 -*-
from zope.component import adapter
from plone.app.ldap.engine.interfaces import ILDAPProperty
from plone.app.ldap.ploneldap.util import guaranteePluginExists
from plone.app.ldap.ploneldap.util import getLDAPPlugin
from plone.app.ldap.ploneldap.util import configureLDAPSchema
from plone.app.ldap.ploneldap.util import addMandatorySchemaItems
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.lifecycleevent.interfaces import IObjectCreatedEvent
from zope.container.interfaces import IObjectRemovedEvent


@adapter(ILDAPProperty, IObjectCreatedEvent)
def HandleCreated(property, event):
    if guaranteePluginExists():
        # A new fully configured plugin has been created, so we do not
        # need to do anything anymore.
        return

    luf=getLDAPPlugin()._getLDAPUserFolder()
    # In case if the user is adding a property which is already present in the
    # backend since it is obligatory we try to delete it first.
    luf.manage_deleteLDAPSchemaItems([str(property.ldap_name)])

    luf.manage_addLDAPSchemaItem(
            ldap_name=str(property.ldap_name),
            friendly_name=property.description,
            public_name=str(property.plone_name),
            multivalued=property.multi_valued,
            binary=property.binary)


@adapter(ILDAPProperty, IObjectModifiedEvent)
def HandleModified(property, event):
    if guaranteePluginExists():
        # A new fully configured plugin has been created, so we do not
        # need to do anything anymore.
        return

    configureLDAPSchema()


@adapter(ILDAPProperty, IObjectRemovedEvent)
def HandleRemoved(property, event):
    if guaranteePluginExists():
        # A new fully configured plugin has been created, so we do not
        # need to do anything anymore.
        return

    luf=getLDAPPlugin()._getLDAPUserFolder()
    luf.manage_deleteLDAPSchemaItems([str(property.ldap_name)])



# -*- coding: utf-8 -*-
from plone.app.ldap.config import PROJECTNAME
from plone.app.ldap.testing import INTEGRATION_TESTING

import unittest


class InstallTestCase(unittest.TestCase):

    """Ensure product is properly installed."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.qi = self.portal['portal_quickinstaller']

    def test_installed(self):
        self.assertTrue(self.qi.isProductInstalled(PROJECTNAME))


class UninstallTestCase(unittest.TestCase):

    """Ensure product is properly uninstalled."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.qi = self.portal['portal_quickinstaller']
        self.qi.uninstallProducts(products=[PROJECTNAME])

    def test_uninstalled(self):
        self.assertFalse(self.qi.isProductInstalled(PROJECTNAME))



# -*- coding: utf-8 -*-
from zope.component.hooks import getSite
from zope.component import getUtility
from zope.interface import directlyProvides
from plone.app.ldap.engine.interfaces import ILDAPConfiguration
from plone.app.ldap.engine.schema import LDAPProperty
from Products.CMFCore.utils import getToolByName
from Products.PloneLDAP.plugins.ad import PloneActiveDirectoryMultiPlugin
from Products.PloneLDAP.plugins.ldap import PloneLDAPMultiPlugin
from Products.PloneLDAP.factory import genericPluginCreation
from plone.app.ldap.ploneldap.interfaces import IManagedLDAPPlugin


def getPAS():
    site=getSite()
    return getToolByName(site, "acl_users")


def guaranteePluginExists():
    """Make sure a proper PAS plugin exists.

    If a plugin exists but it is of the wrong type (AD vs standard LDAP)
    the plugin is removed and a new one is created.

    If a new plugin has been created True is returned. If there was
    a valid plugin present False is returned.
    """
    config=getUtility(ILDAPConfiguration)
    try:
        plugin=getLDAPPlugin()
        if plugin.meta_type==PloneActiveDirectoryMultiPlugin.meta_type and \
                config.ldap_type==u"AD":
            return False
        if plugin.meta_type==PloneLDAPMultiPlugin.meta_type and \
                config.ldap_type==u"LDAP":
            return False

        # There is a managed plugin, but it is of the wrong type.
        pas=getPAS()
        pas.manage_delObjects([plugin.getId()])
    except KeyError:
        pass

    createLDAPPlugin()
    configureLDAPServers()
    configureLDAPSchema()
    return True


def getLDAPPlugin():
    pas=getPAS()
    for plugin in pas.objectValues([PloneActiveDirectoryMultiPlugin.meta_type,
                                    PloneLDAPMultiPlugin.meta_type]):
        if IManagedLDAPPlugin.providedBy(plugin):
            return plugin

    raise KeyError


def createLDAPPlugin(id="ldap-plugin"):
    pas=getPAS()
    config=getUtility(ILDAPConfiguration)
    if config.ldap_type==u"AD":
        klass=PloneActiveDirectoryMultiPlugin
    else:
        klass=PloneLDAPMultiPlugin

    genericPluginCreation(pas, klass,
            id=id,
            title="Plone managed LDAP",
            login_attr=str(config.schema[config.login_attribute].ldap_name),
            uid_attr=str(config.schema[config.userid_attribute].ldap_name),
            rdn_attr=str(config.schema[config.rdn_attribute].ldap_name),
            users_base=config.user_base or "",
            users_scope=config.user_scope,
            groups_base=config.group_base or "",
            groups_scope=config.group_scope,
            binduid=config.bind_dn or "",
            bindpwd=config.bind_password or "",
            encryption=config.password_encryption,
            roles=config.default_user_roles or "",
            local_groups=config.local_groups,
            read_only=config.read_only,
            obj_classes=config.user_object_classes)

    plugin=getattr(pas, id)
    plugin.groupid_attr="cn"
    directlyProvides(plugin, IManagedLDAPPlugin)
    enablePASInterfaces()
    enableCaching(config.cache)
    luf=plugin._getLDAPUserFolder()
    luf.manage_changeProperty('_extra_user_filter', config.extra_user_filter)
    luf.manage_changeProperty('_implicit_mapping', config.implicit_mapping)
    luf.manage_changeProperty('_groups_mappings', config.group_mappings)


def configureLDAPServers():
    luf=getLDAPPlugin()._getLDAPUserFolder()
    config=getUtility(ILDAPConfiguration)

    luf.manage_deleteServers(range(len(luf.getServers())))
    for server in config.servers.values():
        if server.enabled:
            luf.manage_addServer(host=server.server,
                                 port=server.port,
                                 use_ssl=server.connection_type,
                                 conn_timeout=server.connection_timeout,
                                 op_timeout=server.operation_timeout)


def addMandatorySchemaItems():
    config=getUtility(ILDAPConfiguration)

    if config.ldap_type==u"AD":
        required = [("dn", {'description': "Distinguished Name"}),
                    ("objectGUID", {'description': "AD Object GUID",
                            'multi_valued': False, 'binary': True}),
                    ("cn", {'description': "Canonical Name"}),
                    ("sAMAccountName", {'description': "AD User Name"}),
                    ("memberOf", {'description': "Group DNs",
                            'multi_valued': True, 'plone_name': "memberOf"})]
    else:
        required = []

    for attr, args in required:
        if attr not in config.schema:
            config.schema.addItem(LDAPProperty(ldap_name=attr, **args))
        if attr not in config.required_attributes:
            config.required_attributes.append(attr)


def configureLDAPSchema():
    luf=getLDAPPlugin()._getLDAPUserFolder()
    config=getUtility(ILDAPConfiguration)
    addMandatorySchemaItems()

    schema={}
    for property in config.schema.values():
        schema[str(property.ldap_name)]=dict(
                ldap_name=str(property.ldap_name),
                friendly_name=property.description,
                public_name=str(property.plone_name),
                multivalued=property.multi_valued,
                binary=property.binary)
    luf.setSchemaConfig(schema)


def enablePASInterfaces():
    plugin=getLDAPPlugin()
    config=getUtility(ILDAPConfiguration)

    common_interfaces = [
            'IUserEnumerationPlugin',
            'IGroupsPlugin',
            'IGroupEnumerationPlugin',
            'IRoleEnumerationPlugin',
            'IAuthenticationPlugin',
            'ICredentialsResetPlugin',
            'IPropertiesPlugin',
            'IRolesPlugin',
            'IGroupIntrospection',
            ]

    ldap_interfaces = common_interfaces + [
            'IGroupManagement',
            'IUserAdderPlugin',
            'IUserManagement',
            ]

    ad_interfaces = common_interfaces

    if config.activated_plugins:
        plugin.manage_activateInterfaces(config.activated_plugins)
    else:
        if config.ldap_type==u"AD":
            plugin.manage_activateInterfaces(ad_interfaces)
            config.activated_plugins = ad_interfaces
        else:
            plugin.manage_activateInterfaces(ldap_interfaces)
            config.activated_plugins = ldap_interfaces

    plugins=getPAS().plugins
    if "IPropertiesPlugin" in config.activated_plugins:
        iface=plugins._getInterfaceFromName("IPropertiesPlugin")
        for i in range(len(plugins.listPlugins(iface))-1):
            plugins.movePluginsUp(iface, [plugin.getId()])

    if config.ldap_type != u"AD":
        for p in ("IUserAdderPlugin", "IGroupManagement"):
            if p in config.activated_plugins:
                iface=plugins._getInterfaceFromName(p)
                for i in range(len(plugins.listPlugins(iface))-1):
                    plugins.movePluginsUp(iface, [plugin.getId()])



def enableCaching(cache_manager="RAMCache"):
    pas=getPAS()
    if pas.ZCacheable_getManager() is None:
        pas.ZCacheable_setManagerId(manager_id=cache_manager)
    getLDAPPlugin().ZCacheable_setManagerId(manager_id=cache_manager)



# -*- coding: utf-8 -*-
from Acquisition import Implicit
from Acquisition import aq_inner, aq_parent
from zope.component import getMultiAdapter
from zope.formlib.form import action, applyChanges
from zope.event import notify
from zope.i18nmessageid import MessageFactory
from zope.lifecycleevent import ObjectModifiedEvent
from Products.Five import BrowserView
try:  # >= 4.1
    from five.formlib.formbase import AddFormBase
    from five.formlib.formbase import EditFormBase
    AddFormBase, EditFormBase  # pyflakes
except ImportError:  # < 4.1
    from Products.Five.formlib.formbase import AddFormBase
    from Products.Five.formlib.formbase import EditFormBase
from plone.app.form.validators import null_validator

PMF = MessageFactory('plone')


class Adding(Implicit, BrowserView):
    __allow_access_to_unprotected_subobjects__ = True

    contentName = None
    request = None
    context = None

    def nextURL(self):
        return None

    def hasCustomAddView(self):
        return False

    def addingInfo(self):
        return []

    def isSingleMenuItem(self):
        return False


class LDAPAddForm(AddFormBase):
    """Base class for add forms.

    This class has a nextURL method which will return the URL of the LDAP
    management screen and standard form actions.
    """
    fieldset = None

    def nextURL(self):
        parent = aq_parent(aq_inner(self.context))
        url = str(getMultiAdapter((parent, self.request), name=u"absolute_url"))
        if self.fieldset is not None:
            return url + "/@@ldap-controlpanel#fieldsetlegend-" + self.fieldset

        return url + "/@@ldap-controlpanel"

    @action(PMF(u"label_save", default=u"Save"), name=u'save')
    def handle_save_action(self, action, data):
        self.createAndAdd(data)

    @action(PMF(u"label_cancel", default=u"Cancel"),
            validator=null_validator, name=u'cancel')
    def handle_cancel_action(self, action, data):
        nextURL = self.nextURL()
        if nextURL:
            self.request.response.redirect(self.nextURL())
        return ''


class LDAPEditForm(EditFormBase):
    """Base class for edit forms.

    This class has a nextURL method which will return the URL of the LDAP
    management screen and standard form actions.
    """
    fieldset = None

    @action(PMF(u"label_save", default=u"Save"), name=u'save')
    def handle_save_action(self, action, data):
        if applyChanges(self.context, self.form_fields, data, self.adapters):
            notify(ObjectModifiedEvent(self.context))
            self.status = "Changes saved"
        else:
            self.status = "No changes"

        nextURL = self.nextURL()
        if nextURL:
            self.request.response.redirect(self.nextURL())
        return ''

    @action(PMF(u"label_cancel", default=u"Cancel"),
            validator=null_validator, name=u'cancel')
    def handle_cancel_action(self, action, data):
        nextURL = self.nextURL()
        if nextURL:
            self.request.response.redirect(self.nextURL())
        return ''

    def nextURL(self):
        parent = aq_parent(aq_inner(self.context))
        url = str(getMultiAdapter((parent, self.request), name=u"absolute_url"))
        if self.fieldset is not None:
            return url + "/@@ldap-controlpanel#fieldsetlegend-" + self.fieldset

        return url + "/@@ldap-controlpanel"



# -*- coding: utf-8 -*-
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer


class Fixture(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import plone.app.ldap
        self.loadZCML(package=plone.app.ldap)

    def setUpPloneSite(self, portal):
        self.applyProfile(portal, 'plone.app.ldap:ldap')

FIXTURE = Fixture()

INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,),
    name='plone.app.testing:Integration',
)

FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,),
    name='plone.app.testing:Functional',
)



# -*- coding: utf-8 -*-
from zope.interface import Interface
from zope.schema import Bool
from zope.schema import Choice
from zope.schema import Container
from zope.schema import Int
from zope.schema import Password
from zope.schema import ASCIILine
from zope.schema import TextLine
from zope.schema import List

from zope.container.interfaces import IContained
from zope.container.interfaces import IOrderedContainer
from zope.container.interfaces import IContainerNamesContainer
from zope.container.constraints import contains

from plone.app.ldap import LDAPMessageFactory as _

from ldap import SCOPE_SUBTREE


class ILDAPBinding(Interface):
    ldap_type = Choice(
            title=_(u"label_ldap_type",
                default=u"LDAP server type"),
            description=_(u"help_ldap_server_type",
                default=u"Plone supports both Active Directory and standard "
                        u"LDAP servers. For Active Directory the read-only "
                        u"LDAP interface which is enabled for all Active "
                        u"Directory servers can be used."),
            vocabulary="plone.app.ldap.engine.LDAPServerTypes",
            default=u"LDAP",
            required=True)

    rdn_attribute = Choice(
            title=_(u"label_ldap_dn_attribute",
                default=u"rDN attribute"),
            description=_(u"help_ldap_dn_attribute",
                default=u"This is attribute is used to build the "
                        u"distinguished name (DN) for users that are being "
                        u"created in your LDAP directory. This is commonly "
                        u"either the users full name ('cn' property) or the "
                        u"userid ('uid' property)."),
            default="uid",
            vocabulary="plone.app.ldap.engine.LDAPSingleValueAttributes",
            required=True)

    userid_attribute = Choice(
            title=_(u"label_ldap_userid_attribute",
                default=u"User id attribute"),
            description=_(u"help_ldap_userid_attribute",
                default=u"This attribute is used as the userid inside Plone "
                        u"for LDAP users. It has to be unique for all users."),
            default="uid",
            vocabulary="plone.app.ldap.engine.LDAPSingleValueAttributes",
            required=True)

    login_attribute = Choice(
            title=_(u"label_ldap_login_attribute",
                default=u"Login name attribute"),
            description=_(u"help_ldap_login_attribute",
                default=u"The attribute is used as the login name for LDAP "
                        u"users logging into your site. In most cases this "
                        u"should be the same as the user id attribute."),
            default="uid",
            vocabulary="plone.app.ldap.engine.LDAPSingleValueAttributes",
            required=True)

    user_object_classes = ASCIILine(
            title=_(u"label_ldap_user_object_classes",
                default=u"LDAP object classes"),
            description=_(u"help_ldap_user_object_classes",
                default=u"Each object in the LDAP database has a structural "
                        u"object class and optionally several supplemental "
                        u"object classes. These classes define the required "
                        u"and optional properties that can be present on an "
                        u"object. Classes can be entered in a comma seperated "
                        u"list."),
            default="pilotPerson",
            required=True)

    bind_dn = ASCIILine(
            title=_(u"label_ldap_bind_dn",
                default=u"Bind DN"),
            description=_(u"help_ldap_bind_dn",
                default=u"The DN of a manager account in the LDAP directory. "
                        u"This must be allowed to access all user and group "
                        u"information as well as be able to update and create "
                        u"users and groups. Please note that Plone only "
                        u"supports simple binds. SASL is not supported."),
            required=False)

    bind_password = Password(
            title=_(u"label_ldap_bind_password",
                default=u"Bind password"),
            description=_(u"help_ldap_bind_password",
                default=u"Password to use when binding to the LDAP server."),
            required=False)

    user_base = ASCIILine(
            title=_(u"label_ldap_user_base",
                default=u"Base DN for users"),
            description=_(u"help_ldap_user_base",
                default=u"This is the location in your LDAP directory where "
                        u"all users are stored."),
            required=True)

    user_scope = Choice(
            title=_(u"label_ldap_user_scope",
                default=u"Search scope for users"),
            description=_(u"help_ldap_user_scope",
                default=u"The search scope determines where the LDAP server "
                        u"will search for users. With \"one level\" it will "
                        u"only look for users directly in the user base "
                        u"location; \"subtree\" will allow the server to also "
                        u"look in subfolders of the user base location."),
            default=SCOPE_SUBTREE,
            vocabulary="plone.app.ldap.engine.LDAPScopes",
            required=True)

    group_base = ASCIILine(
            title=_(u"label_ldap_group_base",
                default=u"Base DN for groups"),
            description=_(u"help_ldap_group_base",
                default=u"This is the location in your LDAP directory where "
                        u"all groups are stored. There are several options for "
                        u"object class and members possible: the groupOfNames, "
                        u"accessGroup or group object classes can be used with "
                        u"members given in the member property, or the "
                        u"groupOfUniqueNames object class can be used with "
                        u"uniqueMember property. In Active Directory systems "
                        u"only the group object class is supported."),
            required=True)

    group_scope = Choice(
            title=_(u"label_ldap_group_scope",
                default=u"Search scope for groups"),
            description=_(u"help_ldap_group_scope",
                default=u"The search scope determines where the LDAP server "
                        u"will search for groups. With \"one level\" it will "
                        u"only look for groups directly in the group base "
                        u"location; \"subtree\" will allow the server to also "
                        u"look in subfolders of the group base location."),
            default=SCOPE_SUBTREE,
            vocabulary="plone.app.ldap.engine.LDAPScopes",
            required=True)

    password_encryption = Choice(
            title =_(u"label_ldap_user_password_encryption",
                default=u"User password encryption"),
            description=_(u"help_ldap_user_password_encryption",
                default=u"Method of encryption used for user passwords."),
            default="crypt",
            vocabulary="plone.app.ldap.engine.LDAPPasswordEncryption",)

    default_user_roles = ASCIILine(
            title=_(u"label_ldap_default_user_roles",
                default=u"Default user roles"),
            description=_(u"help_ldap_default_user_roles",
                default=u"Default roles for new users."),
            default="Member",
            required=True,)

    read_only = Bool(
            title=_(u"label_ldap_read_only",
                default=u"Read Only"),
            description=_(u"help_ldap_read_only",
                default=u"Control whether Plone should attempt to modify "
                        u"objects and properties on the server."),
            default=False,)


class ILDAPConfiguration(ILDAPBinding):
    """LDAP configuration utility"""

    servers = Container(title=u"LDAP servers",
            description=u"List of LDAP servers that can be used.",
            required=True)

    schema = Container(title=u"LDAP schema",
            description=u"The LDAP schema contains information on used LDAP properties",
            required=True)

    activated_plugins = List(title=_(u"Activated plugins"),
            value_type=ASCIILine(
                title=u"interface",))


class ILDAPServerStorage(IOrderedContainer, IContainerNamesContainer):
    """A storage for rules.
    """

    contains("plone.app.ldap.engine.interfaces.ILDAPServer")


class ILDAPServerConfiguration(Interface):
    """Configuration of an LDAP server.
    """
    enabled = Bool(
            title=_(u"label_ldap_enabled",
                default=u"Active"),
            description=_(u"help_ldap_enabled",
                default=u"Use this LDAP server"),
            default=False,
            required=True)

    server = ASCIILine(
            title=_(u"label_ldap_server",
                default=u"Server"),
            description=_(u"help_ldap_server",
                default="The address or hostname of the LDAP server."),
            default="localhost",
            required=True)

    connection_type = Choice(
            title=_(u"label_ldap_connection_type",
                default=u"Connection type"),
            description=_(u"help_ldap_connection_type",
                default=u""),
            vocabulary="plone.app.ldap.engine.LDAPConnectionTypes",
            default=0,
            required=True)

    connection_timeout = Int(
            title=_(u"label_connection_timeout",
                default=u"Connection timeout"),
            description=_(u"help_connection_timeout",
                default=u"The timeout in seconds to wait for a connection to "
                "the LDAP server to be established."),
            default=5,
            min=-1,
            max=300,
            required=True)

    operation_timeout = Int(
            title=_(u"label_operation_timeout",
                default=u"Operation timeout"),
            description=_(u"help_operation_timeout",
                default=u"The timeout in seconds to wait for an operation such "
                "as a search or update to complete. If no timeout should be "
                "used use -1 as value."),
            default=-1,
            min=-1,
            max=300,
            required=True)


class ILDAPServer(IContained, ILDAPServerConfiguration):
    """A LDAP server
    """


class ILDAPSchema(IOrderedContainer, IContainerNamesContainer):
    """A storage for an LDAP schema.
    """

    contains("plone.app.ldap.engine.interfaces.ILDAPProperty")


class ILDAPPropertyConfiguration(Interface):
    description = TextLine(
            title=_(u"label_property_description",
                default=u"Property description"),
            required=False)

    ldap_name = ASCIILine(
            title=_(u"label_ldap_property",
                default=u"LDAP property name"),
            description=_(u"help_ldap_property",
                default=u"The name of the property as used in the "
                "LDAP directory."),
            required=True)

    plone_name = ASCIILine(
            title=_(u"label_plone_property",
                default= u"Plone property name"),
            description=_(u"help_plone_property",
                default=u"The name of the property as used in the Plone site. "
                        u"If no name is specified, the property will not be "
                        u"visible in Plone but may still be used as the login "
                        u"name, user id, or rDN."),
            required=False)

    multi_valued = Bool(
            title=_(u"label_multi_valued",
                default=u"Multi-valued property"),
            description=_(u"help_multi_valued",
                default=u"Select if this property can have multiple values."),
            required=True)

    binary = Bool(
            title=_(u"label_binary",
                default=u"Binary property"),
            description=_(u"help_binary",
                default=u"Select if this property can have binary values."),
            required=True)


class ILDAPProperty(IContained, ILDAPPropertyConfiguration):
    """A LDAP property
    """



# -*- coding: utf-8 -*-
from zope.browser.interfaces import IAdding


class IServerAdding(IAdding):
    """Marker interface for LDAP server add views.
    """


class IPropertyAdding(IAdding):
    """Marker interface for LDAP property add views.
    """



# -*- coding: utf-8 -*-
from OFS.SimpleItem import SimpleItem
from zope.interface import implementer
from plone.app.ldap.engine.interfaces import ILDAPServer


@implementer(ILDAPServer)
class LDAPServer(SimpleItem):

    __name__ = None
    __parent__ = None

    def __init__(self, server=u"", connection_type=0, connection_timeout=5,
            operation_timeout=-1, enabled=False):
        self.server=server
        self.connection_type=connection_type
        self.connection_timeout=connection_timeout
        self.operation_timeout=operation_timeout
        self.enabled=enabled

    @property
    def port(self):
        if self.connection_type==0:
            return "389"
        elif self.connection_type==1:
            return "636"
        else:
            return "0"



# -*- coding: utf-8 -*-
from zope.component import adapter
from plone.app.ldap.engine.interfaces import ILDAPServer
from plone.app.ldap.ploneldap.util import guaranteePluginExists
from plone.app.ldap.ploneldap.util import getLDAPPlugin
from plone.app.ldap.ploneldap.util import configureLDAPServers
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.lifecycleevent.interfaces import IObjectCreatedEvent
from zope.container.interfaces import IObjectRemovedEvent


def FindServerIndex(luf, server):
    servers=luf.getServers()

    for i in range(len(servers)):
        if servers[i].host==server.server and servers[i].port==server.port:
            return i
    raise KeyError


@adapter(ILDAPServer, IObjectCreatedEvent)
def HandleCreated(server, event):
    if guaranteePluginExists():
        # A new fully configured plugin has been created, so we do not
        # need to do anything anymore.
        return

    if not server.enabled:
        return

    luf=getLDAPPlugin()._getLDAPUserFolder()
    luf.manage_addServer(host=server.server,
                         port=server.port,
                         use_ssl=server.connection_type,
                         conn_timeout=server.connection_timeout,
                         op_timeout=server.operation_timeout)


@adapter(ILDAPServer, IObjectModifiedEvent)
def HandleModified(server, event):
    if guaranteePluginExists():
        # A new fully configured plugin has been created, so we do not
        # need to do anything anymore.
        return

    # We have no good way to determine which servers was edited, so just
    # reload them all
    configureLDAPServers()


@adapter(ILDAPServer, IObjectRemovedEvent)
def HandleRemoved(server, event):
    if guaranteePluginExists():
        # A new fully configured plugin has been created, so we do not
        # need to do anything anymore.
        return

    luf=getLDAPPlugin()._getLDAPUserFolder()
    servers=luf.getServers()

    for i in range(len(servers)):
        if servers[i]['host']==server.server and servers[i]['port']==server.port:
            luf.manage_deleteServers((i,))
            return



# -*- coding: utf-8 -*-
from OFS.SimpleItem import SimpleItem
from zope.interface import implementer
from plone.app.ldap.engine.interfaces import ILDAPProperty


@implementer(ILDAPProperty)
class LDAPProperty(SimpleItem):

    __name__ = None
    __parent__ = None

    def __init__(self, ldap_name=u"", plone_name=u"", description=u"",
                 multi_valued=False, binary=False):
        self.description=description
        self.ldap_name=ldap_name
        self.plone_name=plone_name
        self.multi_valued=multi_valued
        self.binary=binary

    def __setattr__(self, name, value):
        if name in ('ldap_name', 'plone_name', 'description'):
            # Store blank fields as empty strings to avoid exporting a
            # literal 'None' with e.g. str(property.attr).
            value = value or u""
        super(LDAPProperty, self).__setattr__(name, value)



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
import logging


logger = logging.getLogger(__name__)


def uninstall(context):
    """Uninstall script"""
    PLUGIN_ID = "ldap-plugin"
    pas = getToolByName(context, 'acl_users')

    # Remove plugin if it exists.
    if PLUGIN_ID not in pas.objectIds():
        return
    pas._delObject(PLUGIN_ID)
    logger.info('Removed LDAP plugin %s from acl_users.', PLUGIN_ID)



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup


version = '1.4.4'

setup(name='plone.app.ldap',
      version=version,
      description="LDAP control panel for Plone 4.2 and higher",
      long_description=(open("README.rst").read() + "\n" +
                        open("CHANGES.rst").read()),
      classifiers=[
          "Development Status :: 6 - Mature",
          "Framework :: Plone",
          "Framework :: Plone :: 4.2",
          "Framework :: Plone :: 4.3",
          "License :: OSI Approved :: GNU General Public License (GPL)",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.7",
          "Topic :: Software Development :: Libraries :: Python Modules",
          "Topic :: System :: Systems Administration :: Authentication/Directory :: LDAP",
          ],
      keywords='plone ldap',
      author='Wichert Akkerman - Simplon',
      author_email='wichert@simplon.biz',
      maintainer='Kevin Teague',
      maintainer_email='kevin@bud.ca',
      url='https://github.com/plone/plone.app.ldap',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plone', 'plone.app'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'plone.memoize',
          'Products.CMFCore',
          'Products.CMFDefault',
          'Products.GenericSetup >= 1.8.2',
          'Products.PloneLDAP',
          'python-ldap',
          'setuptools',
          'six',
          'zope.component',
          'plone.app.form',
          'five.formlib',
          'zope.formlib',
          'zope.i18nmessageid',
          'zope.interface',
          'zope.lifecycleevent',
          'zope.schema',
      ],
      extras_require={
          'test': [
            'plone.api',
            'plone.app.testing',
            'plone.testing',
          ],
      },
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



# -*- coding: utf-8 -*-
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from five.formlib.formbase import EditForm
from ldap import LDAPError
from plone.app.ldap import LDAPMessageFactory as _
from plone.app.ldap.engine.interfaces import ILDAPBinding
from plone.app.ldap.engine.interfaces import ILDAPConfiguration
from plone.app.ldap.ploneldap.util import getLDAPPlugin
from plone.memoize.instance import memoize
from zope.formlib.interfaces import WidgetInputError
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.event import notify
from zope.formlib.form import FormFields
from zope.formlib.form import action
from zope.formlib.form import applyChanges
from zope.formlib.form import haveInputWidgets
from zope.lifecycleevent import ObjectModifiedEvent
from zope.schema.interfaces import ValidationError
import logging

try:
    from zope.lifecycleevent import ObjectRemovedEvent
    ObjectRemovedEvent  # pyflakes
except:
    from zope.container.contained import ObjectRemovedEvent


class LDAPBindFailure(ValidationError):
    __doc__ = _(u"LDAP server refused your credentials")


logger = logging.getLogger("plone.app.ldap")


def LDAPBindingFactory(context):
    return getUtility(ILDAPConfiguration)


class LDAPControlPanel(EditForm):
    template = ViewPageTemplateFile("controlpanel.pt")

    form_fields = FormFields(ILDAPBinding)
    label = u"LDAP Control Panel"
    description = u"In this control panel you can configure an LDAP connection. You can either use a standard LDAP server or a Microsoft Active Directory Server."
    form_name = u"LDAP Control Panel"

    @action(_("Apply"), condition=haveInputWidgets)
    def handle_edit_actions(self, action, data):
        # Filter out non-required fields that have no value so their
        # existing value is not overwritten. This protects us from
        # overwriting the bind password.
        data = dict([(key, value) for (key, value) in data.iteritems()
                     if value is not None])
        if applyChanges(self.context, self.form_fields, data, self.adapters):
            try:
                notify(ObjectModifiedEvent(self.storage))
            except LDAPError:
                widget=self.widgets.get("bind_dn")

                widget.error=WidgetInputError("bind_dn", widget.label,
                                              LDAPBindFailure("value"))
                self.errors += (widget.error,)
                self.status= _("There were errors")
        return self.request.response.redirect(self.nextURL())

    @action(_(u'label_enable', default=u'Enable'), name=u'EnableServer')
    def handle_enable_server(self, action, data):
        for id in self.request.form.get("serverId", []):
            if id in self.storage.servers:
                server = self.storage.servers[id]
                if server.enabled is False:
                    server.enabled = True
                    notify(ObjectModifiedEvent(server))
        return self.request.response.redirect(self.nextURL())

    @action(_(u'label_disable', default=u'Disable'), name=u'DisableServer')
    def handle_disable_server(self, action, data):
        for id in self.request.form.get("serverId", []):
            if id in self.storage.servers:
                server = self.storage.servers[id]
                if server.enabled is True:
                    server.enabled = False
                    notify(ObjectModifiedEvent(server))
        return self.request.response.redirect(self.nextURL())

    @action(_(u'label_delete', default=u'Delete'), name=u'DeleteServer')
    def handle_delete_server(self, action, data):
        for id in self.request.form.get("serverId", []):
            if id in self.storage.servers:
                notify(ObjectRemovedEvent(self.storage.servers[id]))
                del self.storage.servers[id]
        return self.request.response.redirect(self.nextURL())

    @action(_(u'label_delete_property', default=u'Delete'), name=u'DeleteProperty')
    def handle_delete_property(self, action, data):
        for id in self.request.form.get("propertyId", []):
            if id in self.storage.schema:
                notify(ObjectRemovedEvent(self.storage.schema[id]))
                del self.storage.schema[id]
        return self.request.response.redirect(self.nextURL())

    @action(_(u'label_purge', default=u'Purge'), name=u'Purge')
    def handle_cache_purge(self, action, data):
        luf = getLDAPPlugin()._getLDAPUserFolder()
        luf.manage_reinit()
        self.status = 'User caches cleared'
        return self.request.response.redirect(self.nextURL())

    @action(_(u'label_update_cache_timeouts', default=u'Update Cache Timeouts'), name=u'UpdateCacheTimeouts')
    def handle_update_cache_timeouts(self, action, data):
        luf = getLDAPPlugin()._getLDAPUserFolder()
        for cache_type, cache_value_name in [
                ('authenticated', 'auth_cache_seconds'),
                ('anonymous', 'anon_cache_seconds'),
                ('negative', 'negative_cache_seconds'), ]:
            cache_value = self.request.form['form.' + cache_value_name]
            try:
                cache_value = int(cache_value)
            except ValueError:
                continue
            if cache_value != getattr(self, cache_value_name, None):
                luf.setCacheTimeout(cache_type=cache_type, timeout=cache_value)
                self.status = 'Cache timeout changed'
        return self.request.response.redirect(self.nextURL())

    # cache properties delegate to the LDAPUserFolder instance
    def get_auth_cache_seconds(self):
        try:
            luf = getLDAPPlugin()._getLDAPUserFolder()
        except KeyError:
            return 600
        return luf.getCacheTimeout('authenticated')
    def set_auth_cache_seconds(self, value):
        luf = getLDAPPlugin()._getLDAPUserFolder()
        luf.setCacheTimeout(cache_type='authenticated', timeout=value)
    auth_cache_seconds = property(get_auth_cache_seconds, set_auth_cache_seconds)

    def get_anon_cache_seconds(self):
        try:
            luf = getLDAPPlugin()._getLDAPUserFolder()
        except KeyError:
            return 600
        return luf.getCacheTimeout('anonymous')
    def set_anon_cache_seconds(self, value):
        luf = getLDAPPlugin()._getLDAPUserFolder()
        luf.setCacheTimeout(cache_type='anonymous', timeout=value)
    anon_cache_seconds = property(get_anon_cache_seconds, set_anon_cache_seconds)

    def get_negative_cache_seconds(self):
        try:
            luf = getLDAPPlugin()._getLDAPUserFolder()
        except KeyError:
            return 600
        return luf.getCacheTimeout('negative')
    def set_negative_cache_seconds(self, value):
        luf = getLDAPPlugin()._getLDAPUserFolder()
        luf.setCacheTimeout(cache_type='negative', timeout=value)
    negative_cache_seconds = property(get_negative_cache_seconds, set_negative_cache_seconds)

    def anon_cache(self):
        try:
            luf = getLDAPPlugin()._getLDAPUserFolder()
        except KeyError:
            return []
        users = luf.getUsers(authenticated=0)
        for user in users:
            user.cache_type = 'anonymous'
        return users

    def auth_cache(self):
        try:
            luf = getLDAPPlugin()._getLDAPUserFolder()
        except KeyError:
            return []
        users = luf.getUsers(authenticated=1)
        for user in users:
            user.cache_type = 'authenticated'
        return users

    def nextURL(self):
        url = str(
            getMultiAdapter(
                (self.context, self.request), name=u"absolute_url"
            )
        )
        return url + "/@@ldap-controlpanel#" + self.request.form.get('fieldset_id', '')

    @property
    @memoize
    def storage(self):
        return getUtility(ILDAPConfiguration)

    def servers(self):
        def contype(c):
            if c==0:
                return "LDAP"
            elif c==1:
                return "LDAP over SSL"
            else:
                return "LDAP over IPC"

        return [dict(id=s.__name__,
                     enabled=s.enabled,
                     server=s.server,
                     connection_type=contype(s.connection_type),
                     connection_timeout=s.connection_timeout,
                     operation_timeout=s.operation_timeout)
                for s in self.storage.servers.values()]

    def schema(self):
        storage=self.storage

        def protected(attr):
            return attr.__name__ in storage.required_attributes + [
                    storage.rdn_attribute,
                    storage.userid_attribute,
                    storage.login_attribute]

        return [dict(id=p.__name__,
                     description=p.description,
                     ldap_name=p.ldap_name,
                     plone_name=p.plone_name,
                     multi_valued=p.multi_valued,
                     binary=p.binary,
                     protected=protected(p))
                for p in storage.schema.values()]



# -*- coding: utf-8 -*-
from Products.CMFCore.interfaces import ISiteRoot
from plone.app.ldap import LDAPMessageFactory as _
from plone.app.ldap.browser.baseform import Adding
from plone.app.ldap.browser.baseform import LDAPAddForm
from plone.app.ldap.browser.baseform import LDAPEditForm
from plone.app.ldap.browser.interfaces import IServerAdding
from plone.app.ldap.engine.interfaces import ILDAPConfiguration
from plone.app.ldap.engine.interfaces import ILDAPServerConfiguration
from plone.app.ldap.engine.server import LDAPServer
from zope.container.interfaces import INameChooser
from zope.component import adapts
from zope.component import getUtility
from zope.event import notify
from zope.formlib.form import FormFields
from zope.formlib.form import applyChanges
from zope.interface import implementer
from zope.lifecycleevent import ObjectCreatedEvent
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.traversing.interfaces import ITraversable


@implementer(IServerAdding)
class ServerAdding(Adding):

    def add(self, content):
        """Add the server to the context
        """
        storage = getUtility(ILDAPConfiguration).servers
        chooser = INameChooser(storage)
        storage[chooser.chooseName(None, content)] = content

    def namesAccepted(self):
        return False

    def nameAllowed(self):
        return False


class ServerAddForm(LDAPAddForm):
    """An add form for LDAP servers.
    """
    form_fields = FormFields(ILDAPServerConfiguration)
    label = _(u"Add Server")
    description = _(u"Add an new LDAP or ActiveDirectory server.")
    form_name = _(u"Configure server")
    fieldset = "servers"

    def create(self, data):
        server = LDAPServer()
        applyChanges(server, self.form_fields, data)
        notify(ObjectCreatedEvent(server))
        return server


class ServerEditForm(LDAPEditForm):
    """An edit form for LDAP servers.
    """
    form_fields = FormFields(ILDAPServerConfiguration)
    label = _(u"Edit Server")
    description = _(u"Edit a LDAP or ActiveDirectory server.")
    form_name = _(u"Configure server")
    fieldset = "servers"


@implementer(ITraversable)
class ServerNamespace(object):
    """LDAP server traversing.

    Traversing to portal/++ldapserver++id will traverse to the ldap server and
    return it in the current context, acquisition-wrapped.
    """
    adapts(ISiteRoot, IBrowserRequest)

    def __init__(self, context, request=None):
        self.context=context
        self.request=request

    def traverse(self, name, ignore):
        storage = getUtility(ILDAPConfiguration).servers
        return storage[name].__of__(self.context)



# -*- coding: utf-8 -*-
from zope.component import adapts
from zope.component import getUtility
from zope.interface import implementer
from zope.traversing.interfaces import ITraversable
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.formlib.form import FormFields
from zope.formlib.form import applyChanges
from Products.CMFCore.interfaces import ISiteRoot
from plone.app.ldap import LDAPMessageFactory as _
from plone.app.ldap.browser.interfaces import IPropertyAdding
from plone.app.ldap.engine.interfaces import ILDAPConfiguration
from plone.app.ldap.engine.interfaces import ILDAPPropertyConfiguration
from plone.app.ldap.engine.schema import LDAPProperty
from plone.app.ldap.browser.baseform import LDAPAddForm
from plone.app.ldap.browser.baseform import LDAPEditForm
from plone.app.ldap.browser.baseform import Adding


@implementer(IPropertyAdding)
class PropertyAdding(Adding):

    def add(self, content):
        """Add the property to the schema
        """
        schema=getUtility(ILDAPConfiguration).schema
        schema.addItem(content)

    def namesAccepted(self):
        return False

    def nameAllowed(self):
        return False


class PropertyAddForm(LDAPAddForm):
    """An add form for LDAP properties.
    """
    form_fields = FormFields(ILDAPPropertyConfiguration)
    label = _(u"Add Property")
    description = _(u"Add a LDAP property to the schema.")
    form_name = _(u"Configure property")
    fieldset = "schema"

    def create(self, data):
        property = LDAPProperty()
        applyChanges(property, self.form_fields, data)
        return property


class PropertyEditForm(LDAPEditForm):
    """An edit form for LDAP properties.
    """
    form_fields = FormFields(ILDAPPropertyConfiguration)
    label = _(u"Edit Property")
    description = _(u"Edit a LDAP property.")
    form_name = _(u"Configure property")
    fieldset = "schema"


@implementer(ITraversable)
class SchemaNamespace(object):
    """LDAP schema traversing.
    """
    adapts(ISiteRoot, IBrowserRequest)

    def __init__(self, context, request=None):
        self.context=context
        self.request=request

    def traverse(self, name, ignore):
        schema = getUtility(ILDAPConfiguration).schema
        return schema[name].__of__(self.context)
